@@ -10,23 +10,7 @@ from sklearn.metrics import average_precision_score
 from sklearn.metrics import RocCurveDisplay, PrecisionRecallDisplay
 
 
-class CurveFabric(ABC):
-    def __init__(self, col_score, col_target, name=None):
-        self.col_score = col_score
-        self.col_target = col_target
-        if name:
-            self.name = name
-        else:
-            self.name = self.col_score + ' | ' + self.col_target
-
-    @abstractmethod
-    def fit(self, df):
-        pass
-
-    @abstractmethod
-    def plot(self, ax=None, **kwargs):
-        pass
-
+class ByMonthsCurve:
     def _draw_series_to_bar_plot(self, df_series, ax, label='Count'):
         N = len(df_series.index.values)
         ind = np.arange(N)
@@ -45,10 +29,10 @@ class CurveFabric(ABC):
             plt.xticks(ind, df_series.index.values)
         plt.setp(ax.xaxis.get_majorticklabels(), rotation=90)
 
-    def _draw_series_to_line_plot(self, df_series, ax, label, legend=False):
+    def _draw_series_to_line_plot(self, df_series, ax, label, name, legend=False):
         N = len(df_series.index.values)
         ind = np.arange(N)
-        ax.plot(ind, df_series.values)
+        ax.plot(ind, df_series.values, label=name)
         ax.set_ylabel(label)
         ax.set_ylim(bottom=0, auto=True)
         ax.tick_params('y')
@@ -56,8 +40,26 @@ class CurveFabric(ABC):
             ax.legend(loc="lower right")
 
 
-class DistributionCurve(CurveFabric):
+class CurveFabric(ABC, ByMonthsCurve):
     def __init__(self, col_score, col_target, name=None):
+        self.col_score = col_score
+        self.col_target = col_target
+        if name:
+            self.name = name
+        else:
+            self.name = self.col_score + ' | ' + self.col_target
+
+    @abstractmethod
+    def fit(self, df):
+        pass
+
+    @abstractmethod
+    def plot(self, ax=None, ax_twinx=None, title=None, label=None, **kwargs):
+        pass
+
+
+class DistributionCurve(CurveFabric):
+    def __init__(self, col_score, col_target, name=None, **kwargs):
         super().__init__(col_score, col_target, name=name)
         self.series = None
 
@@ -72,29 +74,49 @@ class DistributionCurve(CurveFabric):
 
 
 class GenGINICurve(CurveFabric):
-    def __init__(self, col_score, col_target, col_generation_deals, name=None):
+    def __init__(self, col_score, col_target, col_generation_deals, name=None, **kwargs):
         super().__init__(col_score, col_target, name=name)
         self.series_auc = None
         self._col_generation_deals = col_generation_deals
         self.count = None
 
     def fit(self, df):
-        self.series_auc = df.groupby(self._col_generation_deals).apply(
-            lambda x: 100 * (2 * roc_auc_score(x[self.col_target], x[self.col_score]) - 1) if len(
-                x[self.col_target].unique()) > 1 else 0)
         self.count = df.groupby(self._col_generation_deals).size()
+        self.series_auc = df[~df[self.col_score].isnull()].groupby(self._col_generation_deals).apply(
+            lambda x: 100 * (2 * roc_auc_score(x[self.col_target], x[self.col_score]) - 1) if len(
+                x[self.col_target].unique()) > 1 else None)
+        self.series_auc = self.series_auc.reindex(index=self.count.index)
         return self
 
     def plot(self, ax=None, title=None, **kwargs):
         ax_twinx = kwargs['ax_twinx']
         self._draw_series_to_bar_plot(self.count, ax, label='Count')
-        self._draw_series_to_line_plot(self.series_auc, ax_twinx, label='GINI')
+        self._draw_series_to_line_plot(self.series_auc, ax_twinx, name=self.name, label='GINI', legend=True)
+        if title:
+            ax.set_title(title, fontsize=14, fontweight='bold')
+
+
+class GenRiskCurve(CurveFabric):
+    def __init__(self, col_score, col_target, col_generation_deals, name=None, **kwargs):
+        super().__init__(col_score, col_target, name=name)
+        self._col_generation_deals = col_generation_deals
+        self.risk = None
+
+    def fit(self, df):
+        self.risk = df.groupby(self._col_generation_deals)[self.col_target].agg(['count', 'mean'])
+        return self
+
+    def plot(self, ax=None, ax_twinx=None, title=None, label=None, name=None, **kwargs):
+        if label is None:
+            label='Mean Target'
+        self._draw_series_to_bar_plot(self.risk['count'], ax, label='Count')
+        self._draw_series_to_line_plot(self.risk['mean'], ax_twinx, label=label, name=name, legend=True)
         if title:
             ax.set_title(title, fontsize=14, fontweight='bold')
 
 
 class MeanProbCurve(CurveFabric):
-    def __init__(self, col_score, col_target, name=None):
+    def __init__(self, col_score, col_target, name=None, **kwargs):
         super().__init__(col_score, col_target, name=name)
         self.pd_deciles = None
         self.pd_bins = None
@@ -107,18 +129,17 @@ class MeanProbCurve(CurveFabric):
             self.col_target].agg(['count', 'mean'])
         return self
 
-    def plot(self, ax=None, title=None, **kwargs):
-        self._draw_series_to_bar_plot(
-            self.mean_prob['count'], ax, label='Count')
-        ax_twinx = ax.twinx()
-        self._draw_series_to_line_plot(
-            self.mean_prob['mean'], ax_twinx, label='Risk')
+    def plot(self, ax=None, ax_twinx=None, title=None, label=None, name=None, **kwargs):
+        if label is None:
+            label='Mean Target'
+        self._draw_series_to_bar_plot(self.mean_prob['count'], ax, label='Count')
+        self._draw_series_to_line_plot(self.mean_prob['mean'], ax_twinx, label=label, name=name, legend=True)
         if title:
             ax.set_title(title, fontsize=14, fontweight='bold')
 
 
 class RocAucCurve(CurveFabric):
-    def __init__(self, col_score, col_target, name=None):
+    def __init__(self, col_score, col_target, name=None, **kwargs):
         super().__init__(col_score, col_target, name=name)
         self.fpr = None
         self.tpr = None
@@ -137,7 +158,7 @@ class RocAucCurve(CurveFabric):
         viz = RocCurveDisplay(
             fpr=self.fpr,
             tpr=self.tpr,
-            roc_auc=self.roc_auc,
+            roc_auc=self.roc_auc*100,
             estimator_name=self.name)
 
         if title:
@@ -147,7 +168,7 @@ class RocAucCurve(CurveFabric):
 
 
 class PrecisionRecallCurve(CurveFabric):
-    def __init__(self, col_score, col_target, name=None):
+    def __init__(self, col_score, col_target, name=None, **kwargs):
         super().__init__(col_score, col_target, name=name)
         self.precision = None
         self.recall = None
